@@ -160,6 +160,8 @@ class _book(object):
     return dict(self.classes)
   def getRef(self,typ,addr):
     return self.refs[(typ,addr)]
+  def delRef(self,typ,addr):
+    del self.refs[(typ,addr)]
   def isRegisteredType(self, typ):
     return typ in self.classes.values()
 
@@ -184,15 +186,23 @@ def keepRef(obj,typ=None,origAddr=None):
   ''' Sometypes, your have to cast a c_void_p, You can keep ref in Ctypes object, 
     they might be transient (if obj == somepointer.contents).'''
   if (typ,origAddr) in __book.refs:
-    if origAddr is not None:
-      origAddr=hex(origAddr)
-    else:
+    # ADDRESS already in refs
+    if origAddr is None:
       origAddr='None'
+    else:
+      origAddr=hex(origAddr)
     if typ is not None:
       log.warning('references already in cache %s/%s'%(typ,origAddr))
+    return
   __book.addRef(obj,typ,origAddr)
   return
 
+def delRef(typ,origAddr):
+  ''' Forget about a Ref..'''
+  if (typ,origAddr) in __book.refs:
+    __book.delRef(typ,origAddr)
+  return
+
 def register(klass):
   #klass.classRef = __register
   #__register[ctypes.POINTER(klass)] = klass
@@ -354,7 +364,7 @@ class LoadableMembers(ctypes.Structure):
       # all case, 
       _attrType=None
       if attrtype not in self.classRef:
-        log.debug("I can't know the size of the basic type behind the %s pointer, it's a pointer to basic type"%(attrname))
+        log.debug("I can't know the size of the basic type behind the %s pointer, it's not a pointer to known registered struct type"%(attrname))
         _attrType=None
       else:
         # test valid address mapping
@@ -379,7 +389,7 @@ class LoadableMembers(ctypes.Structure):
       A c_void_p cannot be load generically, You have to take care of that.
     '''
     attrtype=type(attr)
-    return ( (isPointerType(attr) and ( attrtype in self.classRef) and bool(attr) ) or
+    return ( (isPointerType(attr) and ( attrtype in self.classRef) and bool(attr) and not isFunctionType(attr) ) or
               isStructType(attr)  or isCStringPointer(attr) or
               (isArrayType(attr) and not isBasicTypeArrayType(attr) ) ) # should we iterate on Basictypes ? no
 
@@ -470,7 +480,7 @@ class LoadableMembers(ctypes.Structure):
         log.warning('buffer size was too small for this CString')
       attr.string = txt
       return True
-    else:
+    elif isPointerType(attr): # not functionType, it's not loadable
       _attrname='_'+attrname
       _attrType=self.classRef[attrtype]
       attr_obj_address=getaddress(attr)
@@ -489,11 +499,10 @@ class LoadableMembers(ctypes.Structure):
         attr.contents = ref
         return True
       log.debug("%s %s loading from 0x%lx (is_valid_address: %s)"%(attrname,attr,attr_obj_address, memoryMap ))
-      ##### VALID INSTR.
+      ##### Read the struct in memory and make a copy to play with.
       attr.contents=_attrType.from_buffer_copy(memoryMap.readStruct(attr_obj_address, _attrType ))
-      # save that ref and original addr so we dont need to recopy it later
-      keepRef( attr.contents, _attrType,attr_obj_address)
-      #####
+      # save that validated and loaded ref and original addr so we dont need to recopy it later
+      keepRef( attr.contents, _attrType, attr_obj_address)
       log.debug("%s %s loaded memcopy from 0x%lx to 0x%lx"%(attrname, attr, attr_obj_address, (getaddress(attr))   ))
       # recursive validation checks on new struct
       if not bool(attr):
@@ -502,7 +511,10 @@ class LoadableMembers(ctypes.Structure):
       # go and load the pointed struct members recursively
       if not attr.contents.loadMembers(mappings, maxDepth):
         log.debug('member %s was not loaded'%(attrname))
+        #invalidate the cache ref.
+        delRef( _attrType, attr_obj_address)
         return False
+      return True
     #TATAFN
     return True
   
@@ -521,7 +533,9 @@ class LoadableMembers(ctypes.Structure):
     s=''
     if isStructType(attr):
       s=prefix+'"%s": {\t%s%s},\n'%(field, attr.toString(prefix+'\t'),prefix )  
-    elif isBasicTypeArrayType(attr): ## array of something else than int
+    elif isFunctionType(attr):
+      s=prefix+'"%s": 0x%lx, #(FIELD NOT LOADED)\n'%(field, getaddress(attr) )   # only print address in target space
+    elif isBasicTypeArrayType(attr): ## array of something else than int      
       s=prefix+'"%s": b%s,\n'%(field, repr(array2bytes(attr)) )  
       #s=prefix+'"%s" :['%(field)+','.join(["0x%lx"%(val) for val in attr ])+'],\n'
     elif isArrayType(attr): ## array of something else than int/byte
@@ -563,6 +577,8 @@ class LoadableMembers(ctypes.Structure):
       attr=getattr(self,field)
       if isStructType(attr):
         s+='%s (@0x%lx) : {\t%s}\n'%(field,ctypes.addressof(attr), attr )  
+      elif isFunctionType(attr):
+          s+='%s (@0x%lx) : 0x%lx (FIELD NOT LOADED)\n'%(field,ctypes.addressof(attr), getaddress(attr) )   # only print address in target space
       elif isBasicTypeArrayType(attr):
         try:
           s+='%s (@0x%lx) : %s\n'%(field,ctypes.addressof(attr), repr(array2bytes(attr)) )  
@@ -582,7 +598,12 @@ class LoadableMembers(ctypes.Structure):
         else:
           s+='%s (@0x%lx) : (0x%lx) -> {%s}\n'%(field, ctypes.addressof(attr), getaddress(attr), attr.contents) # use struct printer
       elif isCStringPointer(attr):
-        s+='%s (@0x%lx) : %s (CString) \n'%(field,ctypes.addressof(attr), attr.string)  
+        if not bool(attr) :
+          s+='%s (@0x%lx) : 0x%lx\n'%(field,ctypes.addressof(attr), getaddress(attr) )   # only print address/null
+        elif not is_address_local(attr) :
+          s+='%s (@0x%lx) : 0x%lx (FIELD NOT LOADED)\n'%(field,ctypes.addressof(attr), getaddress(attr) )   # only print address in target space
+        else:
+          s+='%s (@0x%lx) : %s (CString) \n'%(field,ctypes.addressof(attr), attr.string)  
       elif type(attr) is long or type(attr) is int:
         s+='%s : %s\n'%(field, hex(attr) )  
       else:
